;-------------------------------------
; Project : PROSCON.PRJ
;           Assembler routine
;-------------------------------------

PositionRead3CD SEGMENT CODE


EQU		LengthOfBuf		25								;dugo bufora danych w ram
EQU		WMCON				096H							;rejestr specjalny AT89S8252

EXTRN DATA (BUF)

;*************************** DEKLARACJE DOSTPU Z POZIOMU JZYKA C ************************************
PUBLIC PositionRead										;odczyt pooenia enkodera
PUBLIC _EE_SaveBUF										;zapis danych do pamici eeprom
PUBLIC _EE_ReadBUF										;oczyt danych z eeprom
PUBLIC _EE_PosActive										;c=1 pozycja jest "wana", c=0 usunita
PUBLIC _EE_StoreByte										;zapis bajtu z R7 pod adres R4:R5
PUBLIC _EE_GetByte										;odczyt bajtu do R7 spod adresu R6:R7

RSEG PositionRead3CD

;*********************************** OBSUGA ENKODERA 6FX2001-5FS12 ***********************************
;funkcja odczytuje pozycj enkodera
;pozycja zwracana jest jako INT w rejestrach R6:R7 (msb:lsb)
PositionRead:	CLR		A
					ANL		P1,#0F7H						;zerowanie linii P1.3 (zegar)
					MOV		B,#4							;4 bity do odczytu
PR_1:
					ORL		P1,#0CH						;ustawienie linii zegara
					NOP
					MOV		C,P1.2						;przepisanie stanu linii danych do flagi C
					ANL		P1,#0F7H						;zerowanie linii zegara
					CPL		C
					RLC		A								;zapisanie odczytanego bitu na najst.pozycji
					DJNZ		B,PR_1						;czy odczytano ju 4 bity?

;nastpne 8 bitw
					MOV		R6,A							;zapamitanie odczytanej powki bajtu
					CLR		A
					MOV		B,#8							;nastpne 8 bitw do odczytu
PRx_1:
					ORL		P1,#0CH						;ustawienie linii zegara
					NOP
					MOV		C,P1.2						;metoda odczytu j.w.
					ANL		P1,#0F7H
					CPL		C
					RLC		A
					DJNZ		B,PRx_1						;czy odczytano 8 bitw?
					MOV		R7,A							;zapamitanie bajtu po odczycie

					ORL		P1,#0CH
					ANL		P1,#0F7H
					ORL		P1,#0CH

					RET


;*********************************** OBSUGA PAMICI EEPROM ***********************************

;Testowanie bitu gotowoci pamici EEPROM po operacji zapisu
EE_BusyTest:	MOV     	A,WMCON
					ANL     	A,#00000010B
					JZ      	EE_BusyTest
					RET


;Zapamitanie bajtu danych w pamici EEPROM,16-bitowy adres w DPTR,bajt w ACC
EE_SaveByte:	ORL     	WMCON,#00011000B     ;Wczenie trybu zapisu
					MOVX    	@DPTR,A              ;Zapis bajtu
					CALL    	EE_BusyTest          ;Test zakoczenia operacji
					RET


;Odczyt bajtu danych z pamici EEPROM,16-bitowy adres w DPTR,bajt zwracany w ACC
EE_ReadByte:	ANL     	WMCON,#11100111B
					ORL     	WMCON,#00001000B		;Tryb odczytu
					MOVX    	A,@DPTR					;Odczyt bajtu
					RET


;odczyt stanu rekordu w eeprom, status DELETED / VALID zwracany we fladze C (D=0,V=1)
_EE_PosActive:	MOV		DPL,R7					;adres do dptr
					MOV		DPH,R6
					CALL		EE_ReadByte;			;odczyt bajtu statusu
					SETB		C
					CJNE		A,#0FFH,EE_PARet
					CLR		C 
EE_PARet:		ANL     	WMCON,#0E7H				;wyczenie eeprom
					RET


;zapis pojedynczego bajtu do pamici eeprom
_EE_StoreByte:	MOV		A,R7						;<char> przekazywany jest w R7
					MOV		DPL,R5					;adres bajtu int przekazywany w R4:R5
					MOV		DPH,R4
					CALL		EE_SaveByte				;zapis bajtu
					ANL     	WMCON,#0E7H				;wyczenie eeprom
					RET


;odczyt pojedynczego bajtu z eeprom, adres typu INT przekazywany w R6, R7
_EE_GetByte:	MOV		DPL,R7					;zapis adresu bajtu do dptr
					MOV		DPH,R6
					CALL		EE_ReadByte				;odczyt eeprom
					MOV		R7,A						;parametr zwracany jako char
					ANL     	WMCON,#0E7H				;wyczenie eeprom
					RET


;zapis danych do pamici eeprom
_EE_SaveBUF:	MOV		DPL,R7					;odczyt adresu przek.z programu w C
					MOV		DPH,R6
					MOV		B,R5						;liczba bajtw do zapisu
					MOV		R0,#BUF					;do R0 adres obszaru pamici do zapisu
_EE_SaveLoop:	MOV		A,@R0						;pobranie bajtu z BUF
					CALL		EE_SaveByte				;zapis do eeprom
					INC		DPTR						;nastpna pozycja w eeprom
					INC		R0							;nastpna poz.w BUF
					DJNZ		B,_EE_SaveLoop
					ANL		WMCON,#0E7H				;wyczenie eeprom
					RET


;odczyt danych z eeprom
_EE_ReadBUF:	MOV		DPL,R7					;adres w eeprom przek.z C
					MOV		DPH,R6
					MOV		B,R5						;liczba bajtw do odczytu
					MOV		R0,#BUF					;pobranie adresu BUF
_EE_ReadLoop:	CALL		EE_ReadByte				;przepisanie bajtu z pamici eeprom do ram
					MOV		@R0,A
					INC		DPTR						;nastpna poz.w eeprom
					INC		R0
					DJNZ		B,_EE_ReadLoop
					ANL		WMCON,#0E7H				;wyczenie eeprom
					RET


END
